@@ -83,42 +83,44 @@
 /* Function attributes for compiler warnings */
 #ifdef __GNUC__
 # define VLC_DEPRECATED __attribute__((deprecated))
+# if VLC_GCC_VERSION(6,0)
+#  define VLC_DEPRECATED_ENUM __attribute__((deprecated))
+# else
+#  define VLC_DEPRECATED_ENUM
+# endif
 
-# if defined( _WIN32 ) && VLC_GCC_VERSION(4,4)
+# if defined( _WIN32 ) && !defined( __clang__ )
 #  define VLC_FORMAT(x,y) __attribute__ ((format(gnu_printf,x,y)))
 # else
 #  define VLC_FORMAT(x,y) __attribute__ ((format(printf,x,y)))
 # endif
 # define VLC_FORMAT_ARG(x) __attribute__ ((format_arg(x)))
-
 # define VLC_MALLOC __attribute__ ((malloc))
-# define VLC_NORETURN __attribute__ ((noreturn))
-
-# if VLC_GCC_VERSION(3,4)
-#  define VLC_USED __attribute__ ((warn_unused_result))
-# else
-#  define VLC_USED
-# endif
+# define VLC_USED __attribute__ ((warn_unused_result))
 
 #else
 # define VLC_DEPRECATED
+# define VLC_DEPRECATED_ENUM
 # define VLC_FORMAT(x,y)
 # define VLC_FORMAT_ARG(x)
 # define VLC_MALLOC
-# define VLC_NORETURN
 # define VLC_USED
 #endif
 
 
 /* Branch prediction */
 #ifdef __GNUC__
-#   define likely(p)   __builtin_expect(!!(p), 1)
-#   define unlikely(p) __builtin_expect(!!(p), 0)
+# define likely(p)     __builtin_expect(!!(p), 1)
+# define unlikely(p)   __builtin_expect(!!(p), 0)
+# define unreachable() __builtin_unreachable()
 #else
-#   define likely(p)   (!!(p))
-#   define unlikely(p) (!!(p))
+# define likely(p)     (!!(p))
+# define unlikely(p)   (!!(p))
+# define unreachable() ((void)0)
 #endif
 
+#define vlc_assert_unreachable() (assert(!"unreachable"), unreachable())
+
 /* Linkage */
 #ifdef __cplusplus
 # define VLC_EXTERN extern "C"
@@ -128,7 +130,7 @@
 
 #if defined (_WIN32) && defined (DLL_EXPORT)
 # define VLC_EXPORT __declspec(dllexport)
-#elif VLC_GCC_VERSION(4,0)
+#elif defined (__GNUC__)
 # define VLC_EXPORT __attribute__((visibility("default")))
 #else
 # define VLC_EXPORT
@@ -150,7 +152,11 @@
  * epoch). Note that date and time intervals can be manipulated using regular
  * arithmetic operators, and that no special functions are required.
  */
-typedef int64_t mtime_t;
+typedef int64_t vlc_tick_t;
+typedef vlc_tick_t mtime_t; /* deprecated, use vlc_tick_t */
+
+#define VLC_TICK_INVALID  VLC_TS_INVALID
+#define VLC_TICK_0        VLC_TS_0
 
 /**
  * The vlc_fourcc_t type.
@@ -187,9 +193,6 @@ static inline void vlc_fourcc_to_char( vlc_fourcc_t fcc, char *psz_fourcc )
     memcpy( psz_fourcc, &fcc, 4 );
 }
 
-#define vlc_fourcc_to_char( a, b ) \
-        vlc_fourcc_to_char( (vlc_fourcc_t)(a), (char *)(b) )
-
 /*****************************************************************************
  * Classes declaration
  *****************************************************************************/
@@ -202,25 +205,12 @@ typedef struct date_t date_t;
 
 /* Playlist */
 
-/* FIXME */
-/**
- * Playlist commands
- */
-typedef enum {
-    PLAYLIST_PLAY,      /**< No arg.                            res=can fail*/
-    PLAYLIST_VIEWPLAY,  /**< arg1= playlist_item_t*,*/
-                        /**  arg2 = playlist_item_t*          , res=can fail */
-    PLAYLIST_PAUSE,     /**< No arg                             res=can fail*/
-    PLAYLIST_STOP,      /**< No arg                             res=can fail*/
-    PLAYLIST_SKIP,      /**< arg1=int,                          res=can fail*/
-} playlist_command_t;
-
-
 typedef struct playlist_t playlist_t;
 typedef struct playlist_item_t playlist_item_t;
 typedef struct services_discovery_t services_discovery_t;
 typedef struct services_discovery_sys_t services_discovery_sys_t;
-typedef struct playlist_add_t playlist_add_t;
+typedef struct vlc_renderer_discovery_t vlc_renderer_discovery_t;
+typedef struct vlc_renderer_item_t vlc_renderer_item_t;
 
 /* Modules */
 typedef struct module_t module_t;
@@ -232,7 +222,6 @@ typedef struct config_category_t config_category_t;
 typedef struct input_thread_t input_thread_t;
 typedef struct input_item_t input_item_t;
 typedef struct input_item_node_t input_item_node_t;
-typedef struct access_t access_t;
 typedef struct access_sys_t access_sys_t;
 typedef struct stream_t     stream_t;
 typedef struct stream_sys_t stream_sys_t;
@@ -260,6 +249,7 @@ typedef audio_format_t audio_sample_format_t;
 
 /* Video */
 typedef struct vout_thread_t vout_thread_t;
+typedef struct vlc_viewpoint_t vlc_viewpoint_t;
 
 typedef video_format_t video_frame_format_t;
 typedef struct picture_t picture_t;
@@ -304,7 +294,6 @@ typedef struct filter_t filter_t;
 typedef struct filter_sys_t filter_sys_t;
 
 /* Network */
-typedef struct virtual_socket_t v_socket_t;
 typedef struct vlc_url_t vlc_url_t;
 
 /* Misc */
@@ -350,9 +339,7 @@ typedef union
     float           f_float;
     char *          psz_string;
     void *          p_address;
-    vlc_object_t *  p_object;
     vlc_list_t *    p_list;
-    mtime_t         i_time;
     struct { int32_t x; int32_t y; } coords;
 
 } vlc_value_t;
@@ -362,10 +349,9 @@ typedef union
  */
 struct vlc_list_t
 {
-    int             i_count;
-    vlc_value_t *   p_values;
-    int *           pi_types;
-
+    int          i_type;
+    int          i_count;
+    vlc_value_t *p_values;
 };
 
 /*****************************************************************************
@@ -382,7 +368,7 @@ struct vlc_list_t
 #define VLC_ENOITEM        (-8) /**< Item not found */
 
 /*****************************************************************************
- * Variable callbacks
+ * Variable callbacks: called when the value is modified
  *****************************************************************************/
 typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
                                    char const *,            /* variable name */
@@ -390,6 +376,15 @@ typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
                                    vlc_value_t,                 /* new value */
                                    void * );                /* callback data */
 
+/*****************************************************************************
+ * List callbacks: called when elements are added/removed from the list
+ *****************************************************************************/
+typedef int ( * vlc_list_callback_t ) ( vlc_object_t *,      /* variable's object */
+                                        char const *,            /* variable name */
+                                        int,                  /* VLC_VAR_* action */
+                                        vlc_value_t *,      /* new/deleted value  */
+                                        void *);                 /* callback data */
+
 /*****************************************************************************
  * OS-specific headers and thread types
  *****************************************************************************/
@@ -401,8 +396,9 @@ typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
 #   include <windows.h>
 #endif
 
-#ifdef __SYMBIAN32__
- #include <sys/syslimits.h>
+#ifdef __APPLE__
+#include <sys/syslimits.h>
+#include <AvailabilityMacros.h>
 #endif
 
 #ifdef __OS2__
@@ -416,59 +412,82 @@ typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
 #include "vlc_mtime.h"
 #include "vlc_threads.h"
 
-/*****************************************************************************
+/**
  * Common structure members
  *****************************************************************************/
 
-/* VLC_COMMON_MEMBERS : members common to all basic vlc objects */
-#define VLC_COMMON_MEMBERS                                                  \
-/** \name VLC_COMMON_MEMBERS                                                \
- * these members are common for all vlc objects                             \
- */                                                                         \
-/**@{*/                                                                     \
-    const char *psz_object_type;                                            \
-                                                                            \
-    /* Messages header */                                                   \
-    char *psz_header;                                                       \
-    int  i_flags;                                                           \
-                                                                            \
-    /* Object properties */                                                 \
-    bool b_force;      /**< set by the outside (eg. module_need()) */ \
-                                                                            \
-    /* Stuff related to the libvlc structure */                             \
-    libvlc_int_t *p_libvlc;                  /**< (root of all evil) - 1 */ \
-                                                                            \
-    vlc_object_t *  p_parent;                            /**< our parent */ \
-                                                                            \
-/**@}*/                                                                     \
+/**
+ * VLC object common members
+ *
+ * Common public properties for all VLC objects.
+ * Object also have private properties maintained by the core, see
+ * \ref vlc_object_internals_t
+ */
+struct vlc_common_members
+{
+    /** Object type name
+     *
+     * A constant string identifying the type of the object (for logging)
+     */
+    const char *object_type;
 
-/* VLC_OBJECT: attempt at doing a clever cast */
-#if VLC_GCC_VERSION(4,0)
-# ifndef __cplusplus
-#  define VLC_OBJECT( x ) \
-    __builtin_choose_expr( \
-        __builtin_offsetof(__typeof__(*(x)), psz_object_type), \
-        (void)0 /* screw you */, \
-        (vlc_object_t *)(x))
-# else
-#  define VLC_OBJECT( x ) \
-    ((vlc_object_t *)(x) \
-      + 0 * __builtin_offsetof(__typeof__(*(x)), psz_object_type))
-# endif
+    /** Log messages header
+     *
+     * Human-readable header for log messages. This is not thread-safe and
+     * only used by VLM and Lua interfaces.
+     */
+    char *header;
+
+    int  flags;
+
+    /** Module probe flag
+     *
+     * A boolean during module probing when the probe is "forced".
+     * See \ref module_need().
+     */
+    bool force;
+
+    /** LibVLC instance
+     *
+     * Root VLC object of the objects tree that this object belongs in.
+     */
+    libvlc_int_t *libvlc;
+
+    /** Parent object
+     *
+     * The parent VLC object in the objects tree. For the root (the LibVLC
+     * instance) object, this is NULL.
+     */
+    vlc_object_t *parent;
+};
+
+/**
+ * Backward compatibility macro
+ */
+#define VLC_COMMON_MEMBERS struct vlc_common_members obj;
+
+/**
+ * Type-safe vlc_object_t cast
+ *
+ * This macro attempts to cast a pointer to a compound type to a
+ * \ref vlc_object_t pointer in a type-safe manner.
+ * It checks if the compound type actually starts with an embedded
+ * \ref vlc_object_t structure.
+ */
+#if !defined(__cplusplus)
+# define VLC_OBJECT(x) \
+    _Generic((x)->obj, \
+        struct vlc_common_members: (vlc_object_t *)(&(x)->obj), \
+        const struct vlc_common_members: (const vlc_object_t *)(&(x)->obj) \
+    )
 #else
-# define VLC_OBJECT( x ) ((vlc_object_t *)(x))
+# define VLC_OBJECT( x ) ((vlc_object_t *)&(x)->obj)
 #endif
 
 /*****************************************************************************
  * Macros and inline functions
  *****************************************************************************/
 
-/* CEIL: division with round to nearest greater integer */
-#define CEIL(n, d)  ( ((n) / (d)) + ( ((n) % (d)) ? 1 : 0) )
-
-/* PAD: PAD(n, d) = CEIL(n ,d) * d */
-#define PAD(n, d)   ( ((n) % (d)) ? ((((n) / (d)) + 1) * (d)) : (n) )
-
 /* __MAX and __MIN: self explanatory */
 #ifndef __MAX
 #   define __MAX(a, b)   ( ((a) > (b)) ? (a) : (b) )
@@ -502,9 +521,9 @@ static inline uint8_t clip_uint8_vlc( int32_t a )
 
 /** Count leading zeroes */
 VLC_USED
-static inline unsigned clz (unsigned x)
+static inline unsigned (clz)(unsigned x)
 {
-#if VLC_GCC_VERSION(3,4)
+#ifdef __GNUC__
     return __builtin_clz (x);
 #else
     unsigned i = sizeof (x) * 8;
@@ -525,9 +544,9 @@ static inline unsigned clz (unsigned x)
 
 /** Count trailing zeroes */
 VLC_USED
-static inline unsigned ctz (unsigned x)
+static inline unsigned (ctz)(unsigned x)
 {
-#if VLC_GCC_VERSION(3,4)
+#ifdef __GNUC__
     return __builtin_ctz (x);
 #else
     unsigned i = sizeof (x) * 8;
@@ -541,11 +560,12 @@ static inline unsigned ctz (unsigned x)
 #endif
 }
 
+#if !defined(__NetBSD__)
 /** Bit weight */
 VLC_USED
-static inline unsigned popcount (unsigned x)
+static inline unsigned (popcount)(unsigned x)
 {
-#if VLC_GCC_VERSION(3,4)
+#ifdef __GNUC__
     return __builtin_popcount (x);
 #else
     unsigned count = 0;
@@ -558,10 +578,28 @@ static inline unsigned popcount (unsigned x)
 #endif
 }
 
+/** Bit weight of long long */
 VLC_USED
-static inline unsigned parity (unsigned x)
+static inline int (popcountll)(unsigned long long x)
 {
-#if VLC_GCC_VERSION(3,4) && (defined(__amd64__) || defined(__i386__))
+#ifdef __GNUC__
+    return __builtin_popcountll(x);
+#else
+    int count = 0;
+    while (x)
+    {
+        count += x & 1;
+        x = x >> 1;
+    }
+    return count;
+#endif
+}
+#endif
+
+VLC_USED
+static inline unsigned (parity)(unsigned x)
+{
+#ifdef __GNUC__
     return __builtin_parity (x);
 #else
     for (unsigned i = 4 * sizeof (x); i > 0; i /= 2)
@@ -570,24 +608,19 @@ static inline unsigned parity (unsigned x)
 #endif
 }
 
-#ifdef __OS2__
-#   undef bswap16
-#   undef bswap32
-#   undef bswap64
-#endif
-
+#if !defined(__NetBSD__)
 /** Byte swap (16 bits) */
 VLC_USED
-static inline uint16_t bswap16 (uint16_t x)
+static inline uint16_t (bswap16)(uint16_t x)
 {
     return (x << 8) | (x >> 8);
 }
 
 /** Byte swap (32 bits) */
 VLC_USED
-static inline uint32_t bswap32 (uint32_t x)
+static inline uint32_t (bswap32)(uint32_t x)
 {
-#if VLC_GCC_VERSION(4,3) || defined(__clang__)
+#if defined (__GNUC__) || defined(__clang__)
     return __builtin_bswap32 (x);
 #else
     return ((x & 0x000000FF) << 24)
@@ -599,9 +632,9 @@ static inline uint32_t bswap32 (uint32_t x)
 
 /** Byte swap (64 bits) */
 VLC_USED
-static inline uint64_t bswap64 (uint64_t x)
+static inline uint64_t (bswap64)(uint64_t x)
 {
-#if VLC_GCC_VERSION(4,3) || defined(__clang__)
+#if defined (__GNUC__) || defined(__clang__)
     return __builtin_bswap64 (x);
 #elif !defined (__cplusplus)
     return ((x & 0x00000000000000FF) << 56)
@@ -623,7 +656,126 @@ static inline uint64_t bswap64 (uint64_t x)
          | ((x & 0xFF00000000000000ULL) >> 56);
 #endif
 }
+#endif
 
+/* Integer overflow */
+static inline bool uadd_overflow(unsigned a, unsigned b, unsigned *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_uadd_overflow(a, b, res);
+#else
+     *res = a + b;
+     return (a + b) < a;
+#endif
+}
+
+static inline bool uaddl_overflow(unsigned long a, unsigned long b,
+                                  unsigned long *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_uaddl_overflow(a, b, res);
+#else
+     *res = a + b;
+     return (a + b) < a;
+#endif
+}
+
+static inline bool uaddll_overflow(unsigned long long a, unsigned long long b,
+                                   unsigned long long *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_uaddll_overflow(a, b, res);
+#else
+     *res = a + b;
+     return (a + b) < a;
+#endif
+}
+
+#ifndef __cplusplus
+# define add_overflow(a,b,r) \
+    _Generic(*(r), \
+        unsigned: uadd_overflow(a, b, (unsigned *)(r)), \
+        unsigned long: uaddl_overflow(a, b, (unsigned long *)(r)), \
+        unsigned long long: uaddll_overflow(a, b, (unsigned long long *)(r)))
+#else
+static inline bool add_overflow(unsigned a, unsigned b, unsigned *res)
+{
+    return uadd_overflow(a, b, res);
+}
+
+static inline bool add_overflow(unsigned long a, unsigned long b,
+                                unsigned long *res)
+{
+    return uaddl_overflow(a, b, res);
+}
+
+static inline bool add_overflow(unsigned long long a, unsigned long long b,
+                                unsigned long long *res)
+{
+    return uaddll_overflow(a, b, res);
+}
+#endif
+
+#if !(VLC_GCC_VERSION(5,0) || defined(__clang__))
+# include <limits.h>
+#endif
+
+static inline bool umul_overflow(unsigned a, unsigned b, unsigned *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_umul_overflow(a, b, res);
+#else
+     *res = a * b;
+     return b > 0 && a > (UINT_MAX / b);
+#endif
+}
+
+static inline bool umull_overflow(unsigned long a, unsigned long b,
+                                  unsigned long *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_umull_overflow(a, b, res);
+#else
+     *res = a * b;
+     return b > 0 && a > (ULONG_MAX / b);
+#endif
+}
+
+static inline bool umulll_overflow(unsigned long long a, unsigned long long b,
+                                   unsigned long long *res)
+{
+#if VLC_GCC_VERSION(5,0) || defined(__clang__)
+     return __builtin_umulll_overflow(a, b, res);
+#else
+     *res = a * b;
+     return b > 0 && a > (ULLONG_MAX / b);
+#endif
+}
+
+#ifndef __cplusplus
+#define mul_overflow(a,b,r) \
+    _Generic(*(r), \
+        unsigned: umul_overflow(a, b, (unsigned *)(r)), \
+        unsigned long: umull_overflow(a, b, (unsigned long *)(r)), \
+        unsigned long long: umulll_overflow(a, b, (unsigned long long *)(r)))
+#else
+static inline bool mul_overflow(unsigned a, unsigned b, unsigned *res)
+{
+    return umul_overflow(a, b, res);
+}
+
+static inline bool mul_overflow(unsigned long a, unsigned long b,
+                                unsigned long *res)
+{
+    return umull_overflow(a, b, res);
+}
+
+static inline bool mul_overflow(unsigned long long a, unsigned long long b,
+                                unsigned long long *res)
+{
+    return umulll_overflow(a, b, res);
+}
+#endif
 
 /* Free and set set the variable to NULL */
 #define FREENULL(a) do { free( a ); a = NULL; } while(0)
@@ -795,55 +947,29 @@ static inline void SetQWLE (void *p, uint64_t qw)
 #       define O_NONBLOCK 0
 #   endif
 
+/* the mingw32 swab() and win32 _swab() prototypes expect a char* instead of a
+   const void* */
+#  define swab(a,b,c)  swab((char*) (a), (char*) (b), (c))
+
+
 #   include <tchar.h>
 #endif /* _WIN32 */
 
+typedef struct {
+    unsigned num, den;
+} vlc_rational_t;
+
 VLC_API bool vlc_ureduce( unsigned *, unsigned *, uint64_t, uint64_t, uint64_t );
 
-/* Aligned memory allocator */
-#ifdef __APPLE__
-#include <AvailabilityMacros.h>
-#endif
+#define container_of(ptr, type, member) \
+    ((type *)(((char *)(ptr)) - offsetof(type, member)))
 
-#ifdef __MINGW32__
-# define vlc_memalign(align, size) (__mingw_aligned_malloc(size, align))
-# define vlc_free(base)            (__mingw_aligned_free(base))
-#elif defined(_MSC_VER)
-# define vlc_memalign(align, size) (_aligned_malloc(size, align))
-# define vlc_free(base)            (_aligned_free(base))
-#elif defined(__APPLE__) && !defined(MAC_OS_X_VERSION_10_6)
-static inline void *vlc_memalign(size_t align, size_t size)
+VLC_USED VLC_MALLOC
+static inline void *vlc_alloc(size_t count, size_t size)
 {
-    long diff;
-    void *ptr;
-
-    ptr = malloc(size+align);
-    if(!ptr)
-        return ptr;
-    diff = ((-(long)ptr - 1)&(align-1)) + 1;
-    ptr  = (char*)ptr + diff;
-    ((char*)ptr)[-1]= diff;
-    return ptr;
+    return mul_overflow(count, size, &size) ? NULL : malloc(size);
 }
 
-static void vlc_free(void *ptr)
-{
-    if (ptr)
-        free((char*)ptr - ((char*)ptr)[-1]);
-}
-#else
-static inline void *vlc_memalign(size_t align, size_t size)
-{
-    void *base;
-    if (unlikely(posix_memalign(&base, align, size)))
-        base = NULL;
-    return base;
-}
-# define vlc_free(base) free(base)
-#endif
-
-VLC_API void vlc_tdestroy( void *, void (*)(void *) );
-
 /*****************************************************************************
  * I18n stuff
  *****************************************************************************/
@@ -863,26 +989,26 @@ static inline const char *vlc_pgettext_aux( const char *ctx, const char *id )
 /*****************************************************************************
  * Loosy memory allocation functions. Do not use in new code.
  *****************************************************************************/
-static inline void *xmalloc (size_t len)
+static inline void *xmalloc(size_t len)
 {
-    void *ptr = malloc (len);
-    if (unlikely (ptr == NULL))
-        abort ();
+    void *ptr = malloc(len);
+    if (unlikely(ptr == NULL && len > 0))
+        abort();
     return ptr;
 }
 
-static inline void *xrealloc (void *ptr, size_t len)
+static inline void *xrealloc(void *ptr, size_t len)
 {
-    void *nptr = realloc (ptr, len);
-    if (unlikely (nptr == NULL))
-        abort ();
+    void *nptr = realloc(ptr, len);
+    if (unlikely(nptr == NULL && len > 0))
+        abort();
     return nptr;
 }
 
-static inline void *xcalloc (size_t n, size_t size)
+static inline void *xcalloc(size_t n, size_t size)
 {
-    void *ptr = calloc (n, size);
-    if (unlikely (ptr == NULL))
+    void *ptr = calloc(n, size);
+    if (unlikely(ptr == NULL && (n > 0 || size > 0)))
         abort ();
     return ptr;
 }
@@ -911,7 +1037,7 @@ VLC_API const char * VLC_Compiler( void ) VLC_USED;
 #include "vlc_main.h"
 #include "vlc_configuration.h"
 
-#if defined( _WIN32 ) || defined( __SYMBIAN32__ ) || defined( __OS2__ )
+#if defined( _WIN32 ) || defined( __OS2__ )
 #   define DIR_SEP_CHAR '\\'
 #   define DIR_SEP "\\"
 #   define PATH_SEP_CHAR ';'
