@@ -2,7 +2,7 @@
  * vlc_playlist.h : Playlist functions
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: 49dcd1535bdff782f18463b5b45c80b298f8e5c5 $
+ * $Id$
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -24,48 +24,46 @@
 #ifndef VLC_PLAYLIST_H_
 #define VLC_PLAYLIST_H_
 
-#include <vlc_input.h>
-#include <vlc_events.h>
-
 # ifdef __cplusplus
 extern "C" {
 # endif
 
+#include <vlc_events.h>
+#include <vlc_aout.h>
+
 TYPEDEF_ARRAY(playlist_item_t*, playlist_item_array_t)
 
 struct intf_thread_t;
 
 /**
+ * \defgroup playlist VLC playlist
+ * VLC playlist controls
+ * @{
  * \file
- * This file contain structures and function prototypes related
- * to the playlist in vlc
- *
- * \defgroup vlc_playlist Playlist
+ * VLC playlist control interface
  *
  * The VLC playlist system has a tree structure. This allows advanced
  * categorization, like for SAP streams (which are grouped by "sap groups").
  *
- * The base structure for all playlist operations is the input_item_t. This
- * contains all information needed to play a stream and get info, ie, mostly,
- * mrl and metadata. This structure contains a unique i_id field. ids are
- * not recycled when an item is destroyed.
+ * The base structure for all playlist operations is the playlist_item_t.
+ * This is essentially a node within the playlist tree. Each playlist item
+ * references an input_item_t which contains the input stream info, such as
+ * location, name and meta-data.
  *
- * Input items are not used directly, but through playlist items.
- * The playlist items are themselves in a tree structure. They only contain
- * a link to the input item, a unique id and a few flags. the playlist
- * item id is NOT the same as the input item id.
- * Several playlist items can be attached to a single input item. The input
- * item is refcounted and is automatically destroyed when it is not used
- * anymore.
+ * A playlist item is uniquely identified by its input item:
+ * \ref playlist_ItemGetByInput(). A single input item cannot be used by more
+ * than one playlist item at a time; if necessary, a copy of the input item can
+ * be made instead.
  *
- * The top-level items are the main media sources and include:
- * playlist, media library, SAP, Shoutcast, devices, ...
+ * The same playlist tree is visible to all user interfaces. To arbitrate
+ * access, a lock is used, see \ref playlist_Lock() and \ref playlist_Unlock().
  *
- * It is envisioned that a third tree will appear: VLM, but it's not done yet
- *
- * The playlist also stores, for utility purposes, an array of all input
- * items, an array of all playlist items and an array of all playlist items
- * and nodes (both are represented by the same structure).
+ * Under the playlist root item node, the top-level items are the main
+ * media sources and include:
+ * - the actual playlist,
+ * - the media library,
+ * - the service discovery root node, whose children are services discovery
+ *   module instances.
  *
  * So, here is an example:
  * \verbatim
@@ -82,51 +80,44 @@ struct intf_thread_t;
  * \endverbatim
  *
  * Sometimes, an item creates subitems. This happens for the directory access
- * for example. In that case, if the item is under the "playlist" top-level item
- * and playlist is configured to be flat then the item will be deleted and
+ * for example. In that case, if the item is under the "playlist" top-level
+ * item and playlist is configured to be flat then the item will be deleted and
  * replaced with new subitems. If the item is under another top-level item, it
  * will be transformed to a node and removed from the list of all items without
  * nodes.
  *
- * For "standard" item addition, you can use playlist_Add, playlist_AddExt
- * (more options) or playlist_AddInput if you already created your input
+ * For "standard" item addition, you can use playlist_Add(), playlist_AddExt()
+ * (more options) or playlist_AddInput() if you already created your input
  * item. This will add the item at the root of "Playlist" or of "Media library"
  * in each of the two trees.
  *
- * You can create nodes with playlist_NodeCreate and can create items from
- * existing input items to be placed under any node with playlist_NodeAddInput.
- *
- * To delete an item, use playlist_DeleteFromInput( p_item ) which will
- * remove all occurrences of the input.
+ * You can create nodes with playlist_NodeCreate() and can create items from
+ * existing input items to be placed under any node with
+ * playlist_NodeAddInput().
  *
+ * To delete an item, use playlist_NodeDelete( p_item ).
  *
  * The playlist defines the following event variables:
  *
- * - "item-change": It will contain the input_item_t->i_id of a changed input
- * item monitored by the playlist.
- * item being played.
+ * - "item-change": It will contain a pointer to the input_item_t of a
+ * changed input item monitored by the playlist.
  *
- * - "playlist-item-append": It will contain a pointer to a playlist_add_t.
- * - "playlist-item-deleted": It will contain the playlist_item_t->i_id of a
- * deleted playlist_item_t.
+ * - "playlist-item-append": It will contain a pointer to a playlist_item_t.
+ * - "playlist-item-deleted": It will contain a pointer to the playlist_item_t
+ * about to be deleted.
  *
  * - "leaf-to-parent": It will contain the playlist_item_t->i_id of an item that is transformed
  *   into a node.
  *
- * The playlist contains rate-variable which is propagated to current input if available
- * also rate-slower/rate-faster is in use
- *
- * XXX Be really carefull, playlist_item_t->i_id and input_item_t->i_id are not
- * the same. Yes, the situation is pretty bad.
- *
- * @{
+ * The playlist contains rate-variable which is propagated to current input if
+ * available also rate-slower/rate-faster is in use.
  */
 
 /** Helper structure to export to file part of the playlist */
 typedef struct playlist_export_t
 {
     VLC_COMMON_MEMBERS
-    const char *psz_filename;
+    char *base_url;
     FILE *p_file;
     playlist_item_t *p_root;
 } playlist_export_t;
@@ -139,21 +130,17 @@ struct playlist_item_t
     playlist_item_t      **pp_children; /**< Children nodes/items */
     playlist_item_t       *p_parent;    /**< Item parent */
     int                    i_children;  /**< Number of children, -1 if not a node */
+    unsigned               i_nb_played; /**< Times played */
 
     int                    i_id;        /**< Playlist item specific id */
     uint8_t                i_flags;     /**< Flags \see playlist_item_flags_e */
-
-    playlist_t            *p_playlist;  /**< Parent playlist */
 };
 
 typedef enum {
-    PLAYLIST_SAVE_FLAG         = 0x0001,  /**< Must it be saved */
-    PLAYLIST_SKIP_FLAG         = 0x0002,  /**< Must playlist skip after it ? */
-    PLAYLIST_DBL_FLAG          = 0x0004,  /**< Is it disabled ? */
-    PLAYLIST_RO_FLAG           = 0x0008,  /**< Write-enabled ? */
-    PLAYLIST_REMOVE_FLAG       = 0x0010,  /**< Remove this item at the end */
-    PLAYLIST_EXPANDED_FLAG     = 0x0020,  /**< Expanded node */
-    PLAYLIST_SUBITEM_STOP_FLAG = 0x0040,  /**< Must playlist stop if the item gets subitems ?*/
+    PLAYLIST_DBL_FLAG          = 0x04,  /**< Is it disabled ? */
+    PLAYLIST_RO_FLAG           = 0x08,  /**< Write-enabled ? */
+    PLAYLIST_SUBITEM_STOP_FLAG = 0x40,  /**< Must playlist stop if the item gets subitems ?*/
+    PLAYLIST_NO_INHERIT_FLAG   = 0x80,  /**< Will children inherit flags the R/O flag ? */
 } playlist_item_flags_e;
 
 /** Playlist status */
@@ -166,30 +153,14 @@ struct playlist_t
     VLC_COMMON_MEMBERS
 
     playlist_item_array_t items; /**< Arrays of items */
-    playlist_item_array_t all_items; /**< Array of items and nodes */
 
     playlist_item_array_t current; /**< Items currently being played */
     int                   i_current_index; /**< Index in current array */
 
     /* Predefined items */
-    playlist_item_t *     p_root;
-    playlist_item_t *     p_playing;
-    playlist_item_t *     p_media_library;
-
-    //Phony ones, point to those above;
-    playlist_item_t *     p_root_category; /**< Root of category tree */
-    playlist_item_t *     p_root_onelevel; /**< Root of onelevel tree */
-    playlist_item_t *     p_local_category; /** < "Playlist" in CATEGORY view */
-    playlist_item_t *     p_ml_category; /** < "Library" in CATEGORY view */
-    playlist_item_t *     p_local_onelevel; /** < "Playlist" in ONELEVEL view */
-    playlist_item_t *     p_ml_onelevel; /** < "Library" in ONELEVEL view */
-};
-
-/** Helper to add an item */
-struct playlist_add_t
-{
-    int i_node; /**< Playist id of the parent node */
-    int i_item; /**< Playist id of the playlist_item_t */
+    playlist_item_t  root;
+    playlist_item_t *p_playing;
+    playlist_item_t *p_media_library;
 };
 
 /* A bit of macro magic to generate an enum out of the following list,
@@ -208,7 +179,9 @@ struct playlist_add_t
     DEF( SORT_TRACK_NUMBER )\
     DEF( SORT_DESCRIPTION )\
     DEF( SORT_RATING )\
-    DEF( SORT_URI )
+    DEF( SORT_URI )\
+    DEF( SORT_DISC_NUMBER )\
+    DEF( SORT_DATE )
 
 #define DEF( s ) s,
 enum
@@ -228,15 +201,7 @@ enum
     ORDER_REVERSE = 1,
 };
 
-/* Used by playlist_Import */
-#define PLAYLIST_INSERT          0x0001
-#define PLAYLIST_APPEND          0x0002
-#define PLAYLIST_GO              0x0004
-#define PLAYLIST_PREPARSE        0x0008
-#define PLAYLIST_SPREPARSE       0x0010
-#define PLAYLIST_NO_REBUILD      0x0020
-
-#define PLAYLIST_END           -666
+#define PLAYLIST_END           -1
 
 enum pl_locked_state
 {
@@ -253,16 +218,78 @@ enum pl_locked_state
 #define PL_UNLOCK playlist_Unlock( p_playlist )
 #define PL_ASSERT_LOCKED playlist_AssertLocked( p_playlist )
 
-/* Playlist control */
+/** Playlist commands */
+enum {
+    PLAYLIST_PLAY,      /**< No arg.                            res=can fail*/
+    PLAYLIST_VIEWPLAY,  /**< arg1= playlist_item_t*,*/
+                        /**  arg2 = playlist_item_t*          , res=can fail */
+    PLAYLIST_TOGGLE_PAUSE, /**< No arg                          res=can fail */
+    PLAYLIST_STOP,      /**< No arg                             res=can fail*/
+    PLAYLIST_SKIP,      /**< arg1=int,                          res=can fail*/
+    PLAYLIST_PAUSE,     /**< No arg */
+    PLAYLIST_RESUME,    /**< No arg */
+};
+
 #define playlist_Play(p) playlist_Control(p,PLAYLIST_PLAY, pl_Unlocked )
-#define playlist_Pause(p) playlist_Control(p,PLAYLIST_PAUSE, pl_Unlocked )
+#define playlist_TogglePause(p) \
+        playlist_Control(p, PLAYLIST_TOGGLE_PAUSE, pl_Unlocked)
 #define playlist_Stop(p) playlist_Control(p,PLAYLIST_STOP, pl_Unlocked )
 #define playlist_Next(p) playlist_Control(p,PLAYLIST_SKIP, pl_Unlocked, 1)
 #define playlist_Prev(p) playlist_Control(p,PLAYLIST_SKIP, pl_Unlocked, -1)
 #define playlist_Skip(p,i) playlist_Control(p,PLAYLIST_SKIP, pl_Unlocked,  (i) )
+#define playlist_Pause(p) \
+        playlist_Control(p, PLAYLIST_PAUSE, pl_Unlocked)
+#define playlist_Resume(p) \
+        playlist_Control(p, PLAYLIST_RESUME, pl_Unlocked)
 
+/**
+ * Locks the playlist.
+ *
+ * This function locks the playlist. While the playlist is locked, no other
+ * thread can modify the playlist tree layout or current playing item and node.
+ *
+ * Locking the playlist is necessary before accessing, either for reading or
+ * writing, any playlist item.
+ *
+ * \note Because of the potential for lock inversion / deadlocks, locking the
+ * playlist shall not be attemped while holding an input item lock. An input
+ * item lock can be acquired while holding the playlist lock.
+ *
+ * While holding the playlist lock, a thread shall not attempt to:
+ * - probe, initialize or deinitialize a module or a plugin,
+ * - install or deinstall a variable or event callback,
+ * - set a variable or trigger a variable callback, with the sole exception
+ *   of the playlist core triggering add/remove/leaf item callbacks,
+ * - invoke a module/plugin callback other than:
+ *   - playlist export,
+ *   - logger message callback.
+ */
 VLC_API void playlist_Lock( playlist_t * );
+
+/**
+ * Unlocks the playlist.
+ *
+ * This function unlocks the playlist, allowing other threads to lock it. The
+ * calling thread must have called playlist_Lock() before.
+ *
+ * This function invalidates all or any playlist item pointers.
+ * There are no ways to ensure that playlist items are not modified or deleted
+ * by another thread past this function call.
+ *
+ * To retain a reference to a playlist item while not holding the playlist
+ * lock, a thread should take a reference to the input item within the
+ * playlist item before unlocking. If this is not practical, then the thread
+ * can store the playlist item ID (i_id) before unlocking.
+ * Either way, this will not ensure that the playlist item is not deleted, so
+ * the thread must be ready to handle that case later when calling
+ * playlist_ItemGetByInput() or playlist_ItemGetById().
+ *
+ * Furthermore, if ID is used, then the playlist item might be deleted, and
+ * another item could be assigned the same ID. To avoid that problem, use
+ * the input item instead of the ID.
+ */
 VLC_API void playlist_Unlock( playlist_t * );
+
 VLC_API void playlist_AssertLocked( playlist_t * );
 VLC_API void playlist_Deactivate( playlist_t * );
 
@@ -274,17 +301,23 @@ VLC_API void playlist_Deactivate( playlist_t * );
  * \param i_query the command to do
  * \param b_locked TRUE if playlist is locked when entering this function
  * \param variable number of arguments
- * \return VLC_SUCCESS or an error
  */
-VLC_API int playlist_Control( playlist_t *p_playlist, int i_query, bool b_locked, ...  );
+VLC_API void playlist_Control( playlist_t *p_playlist, int i_query, int b_locked, ...  );
+
+static inline void playlist_ViewPlay(playlist_t *pl, playlist_item_t *node,
+                                     playlist_item_t *item)
+{
+    playlist_Control(pl, PLAYLIST_VIEWPLAY, pl_Locked, node, item);
+}
 
 /** Get current playing input. The object is retained.
  */
 VLC_API input_thread_t * playlist_CurrentInput( playlist_t *p_playlist ) VLC_USED;
+VLC_API input_thread_t *playlist_CurrentInputLocked( playlist_t *p_playlist ) VLC_USED;
 
 /** Get the duration of all items in a node.
  */
-VLC_API mtime_t playlist_GetNodeDuration( playlist_item_t * );
+VLC_API vlc_tick_t playlist_GetNodeDuration( playlist_item_t * );
 
 /** Clear the playlist
  * \param b_locked TRUE if playlist is locked when entering this function
@@ -301,13 +334,13 @@ VLC_API int playlist_Status( playlist_t * );
 
 /**
  * Export a node of the playlist to a certain type of playlistfile
- * \param p_playlist the playlist to export
+ * \param b_playlist true for the playlist, false for the media library
  * \param psz_filename the location where the exported file will be saved
- * \param p_export_root the root node to export
  * \param psz_type the type of playlist file to create (m3u, pls, ..)
  * \return VLC_SUCCESS on success
  */
-VLC_API int playlist_Export( playlist_t *p_playlist, const char *psz_name, playlist_item_t *p_export_root, const char *psz_type );
+VLC_API int playlist_Export( playlist_t *p_playlist, const char *psz_name,
+                             bool b_playlist, const char *psz_type );
 
 /**
  * Open a playlist file, add its content to the current playlist
@@ -316,7 +349,7 @@ VLC_API int playlist_Import( playlist_t *p_playlist, const char *psz_file );
 
 /********************** Services discovery ***********************/
 
-/** Add a list of comma-separated service discovery modules */
+/** Add a service discovery module */
 VLC_API int playlist_ServicesDiscoveryAdd(playlist_t *, const char *);
 /** Remove a services discovery module by name */
 VLC_API int playlist_ServicesDiscoveryRemove(playlist_t *, const char *);
@@ -325,25 +358,32 @@ VLC_API bool playlist_IsServicesDiscoveryLoaded( playlist_t *,const char *) VLC_
 /** Query a services discovery */
 VLC_API int playlist_ServicesDiscoveryControl( playlist_t *, const char *, int, ... );
 
+/********************** Renderer ***********************/
+/**
+ * Sets a renderer or remove the current one
+ * @param p_item    The renderer item to be used, or NULL to disable the current
+ *                  one. If a renderer is provided, its reference count will be
+ *                  incremented.
+ */
+VLC_API int playlist_SetRenderer( playlist_t* p_pl, vlc_renderer_item_t* p_item );
 
 
 /********************************************************
  * Item management
  ********************************************************/
 
-/*************************** Item deletion **************************/
-VLC_API int playlist_DeleteFromInput( playlist_t *, input_item_t *, bool );
-
 /******************** Item addition ********************/
-VLC_API int playlist_Add( playlist_t *, const char *, const char *, int, int, bool, bool );
-VLC_API int playlist_AddExt( playlist_t *, const char *, const char *, int, int, mtime_t, int, const char *const *, unsigned, bool, bool );
-VLC_API int playlist_AddInput( playlist_t *, input_item_t *, int, int, bool, bool );
-VLC_API playlist_item_t * playlist_NodeAddInput( playlist_t *, input_item_t *, playlist_item_t *, int, int, bool );
+VLC_API int playlist_Add( playlist_t *, const char *, bool );
+VLC_API int playlist_AddExt( playlist_t *, const char *, const char *, bool, int, const char *const *, unsigned, bool );
+VLC_API int playlist_AddInput( playlist_t *, input_item_t *, bool, bool );
+VLC_API playlist_item_t * playlist_NodeAddInput( playlist_t *, input_item_t *, playlist_item_t *, int );
 VLC_API int playlist_NodeAddCopy( playlist_t *, playlist_item_t *, playlist_item_t *, int );
 
 /********************************** Item search *************************/
 VLC_API playlist_item_t * playlist_ItemGetById(playlist_t *, int ) VLC_USED;
-VLC_API playlist_item_t * playlist_ItemGetByInput(playlist_t *,input_item_t * ) VLC_USED;
+VLC_API playlist_item_t *playlist_ItemGetByInput(playlist_t *,
+                                                 const input_item_t * )
+VLC_USED;
 
 VLC_API int playlist_LiveSearchUpdate(playlist_t *, playlist_item_t *, const char *, bool );
 
@@ -351,15 +391,9 @@ VLC_API int playlist_LiveSearchUpdate(playlist_t *, playlist_item_t *, const cha
  * Tree management
  ********************************************************/
 /* Node management */
-VLC_API playlist_item_t * playlist_NodeCreate( playlist_t *, const char *, playlist_item_t * p_parent, int i_pos, int i_flags, input_item_t * );
-VLC_API int playlist_NodeAppend(playlist_t *,playlist_item_t*,playlist_item_t *);
-VLC_API int playlist_NodeInsert(playlist_t *,playlist_item_t*,playlist_item_t *, int);
-VLC_API int playlist_NodeRemoveItem(playlist_t *,playlist_item_t*,playlist_item_t *);
+VLC_API playlist_item_t * playlist_NodeCreate( playlist_t *, const char *, playlist_item_t * p_parent, int i_pos, int i_flags );
 VLC_API playlist_item_t * playlist_ChildSearchName(playlist_item_t*, const char* ) VLC_USED;
-VLC_API int playlist_NodeDelete( playlist_t *, playlist_item_t *, bool , bool );
-
-VLC_API playlist_item_t * playlist_GetNextLeaf( playlist_t *p_playlist, playlist_item_t *p_root, playlist_item_t *p_item, bool b_ena, bool b_unplayed ) VLC_USED;
-VLC_API playlist_item_t * playlist_GetPrevLeaf( playlist_t *p_playlist, playlist_item_t *p_root, playlist_item_t *p_item, bool b_ena, bool b_unplayed ) VLC_USED;
+VLC_API void playlist_NodeDelete( playlist_t *, playlist_item_t * );
 
 /**************************
  * Audio output management
@@ -367,9 +401,6 @@ VLC_API playlist_item_t * playlist_GetPrevLeaf( playlist_t *p_playlist, playlist
 
 VLC_API audio_output_t *playlist_GetAout( playlist_t * );
 
-#define AOUT_VOLUME_DEFAULT             256
-#define AOUT_VOLUME_MAX                 512
-
 VLC_API float playlist_VolumeGet( playlist_t * );
 VLC_API int playlist_VolumeSet( playlist_t *, float );
 VLC_API int playlist_VolumeUp( playlist_t *, int, float * );
